// $Id: CaptureConsole.java,v 1.1 2001/05/17 20:15:36 pcharles Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.client;

import java.util.Collection;
import java.util.HashSet;
import net.ultrametrics.console.TestConsole;
import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.capture.PacketCapture;
import net.sourceforge.jpcap.capture.CapturePacketException;
import net.sourceforge.jpcap.capture.InvalidFilterException;
import net.sourceforge.jpcap.capture.CaptureDeviceOpenException;


/**
 * Text console for controlling packet capture.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @version $Revision: 1.1 $
 * @lastModifiedBy $Author: pcharles $
 * @lastModifiedAt $Date: 2001/05/17 20:15:36 $
 */
public class CaptureConsole extends TestConsole
{
  /**
   * Construct a new capture console.
   * @param captureTool the capture tool service handle.
   * @param captureSystem the packet capture system.
   */
  public CaptureConsole(CaptureTool captureTool, 
                        PacketCapture captureSystem) {
    this.captureTool = captureTool;
    this.captureSystem = captureSystem;
  }

  /**
   * Command processor. Handles commands specific to packet capture.
   * Invokes parent to handle commands inherited from other consoles.
   * @param command the command to execute.
   * @param args optional and required arguments to the provided command.
   */
  public String doCommand(String command, String [] args) {
    String result = "";
    if(command.toLowerCase().startsWith("device"))
      return command_device(args);
    if(command.toLowerCase().startsWith("snaplen"))
      return command_snaplen(args);
    if(command.toLowerCase().startsWith("timeout"))
      return command_timeout(args);
    if(command.toLowerCase().startsWith("filter"))
      return command_filter(args);
    if(command.toLowerCase().startsWith("status"))
      return command_status();
    if(command.toLowerCase().startsWith("capture"))
      return command_capture(args);

    result = super.doCommand(command, args);

    return result;
  }

  /**
   * Set the device name.
   */
  public String command_device(String [] args) {
    if(args.length < 1) 
      return "usage: device device_name";

    captureTool.setDevice(args[0]);
    return "device set to '" + args[0] + "'";
  }

  /**
   * Set the capture data length.
   */
  public String command_snaplen(String [] args) {
    if(args.length < 1) 
      return "usage: snaplen length";

    snaplen = Integer.parseInt(args[0]);
    return "snaplen set to " + snaplen;
  }

  /**
   * Set the capture timeout in ms.
   */
  public String command_timeout(String [] args) {
    if(args.length < 1) 
      return "usage: timeout length";

    timeout = Integer.parseInt(args[0]);
    return "timeout set to " + timeout;
  }

  /**
   * Set the filter expression.
   */
  public String command_filter(String [] args) {
    if(args.length < 1) 
      return "usage: filter filter_expression";
    filter = "";
    for(int i=0; i<args.length; i++)
      filter = filter + " " + args[i];

    return "filter set to '" + filter + "'";
  }

  /**
   * show the status.
   */
  public String command_status() {
    StringBuffer buffer = new StringBuffer();
    buffer.append("\tdevice = '" + captureTool.getDevice() + "'\n");
    buffer.append("\tsnaplen = " + snaplen + "\n");
    buffer.append("\ttimeout = " + timeout + "\n");
    buffer.append("\tfilter = '" + filter + "'\n");
    buffer.append("\t" + captureSystem.getStatistics() + "\n");

    return buffer.toString();
  }

  /**
   * Capture packets.
   */
  public String command_capture(String [] args) {
    if(args.length < 1) 
      return "usage: capture packet_count";

    int count = Integer.parseInt(args[0]);
    try {
      captureSystem.open(captureTool.getDevice(), snaplen, 
                         promiscuous, timeout);
      captureSystem.setFilter(filter, true);
      captureSystem.capture(count);
    }
    catch(CaptureDeviceOpenException cdoe) {
      System.err.println(cdoe);
    }
    catch(InvalidFilterException ife) {
      System.err.println(ife);
    }
    catch(CapturePacketException cpe) {
      System.err.println(cpe);
    }

    return "capture complete";
  }


  /**
   * Start capturing packets.
   */
  public String command_start() {

    return "";
  }

  /**
   * Generate help for commands specific to this console.
   * Invoke parent to give help on commands inherited from other consoles.
   * @param suppressed a collection of commands handled by child consoles.
   */
  protected String command_help(Collection suppressed) {
    StringBuffer buffer = new StringBuffer();
    
    // suppress help display of commands reimplemented here
    HashSet suppress = new HashSet();
    if(suppressed == null) 
      suppressed = new HashSet();

    suppress.addAll(suppressed);
    buffer.append(super.command_help(suppress));

    buffer.append("  capture:\n");
    if(! suppressed.contains("device"))
      buffer.append("\tdevice [device]         - " + 
                    "set name of the device to capture packets on\n");
    if(! suppressed.contains("snaplen"))
      buffer.append("\tsnaplen [ms]            - " + 
                    "set the capture timeout in milliseconds\n");
    if(! suppressed.contains("timeout"))
      buffer.append("\ttimeout [length]        - " + 
                    "set the capture data length in bytes\n");
    if(! suppressed.contains("filter"))
      buffer.append("\tfilter [expression]     - " + 
                    "set the filter expression\n");
    if(! suppressed.contains("status"))
      buffer.append("\tstatus                  - " + 
                    "show current capture status\n");
    if(! suppressed.contains("capture"))
      buffer.append("\tcapture [count]         - " + 
                    "capture packets\n");
    if(! suppressed.contains("dump"))
      buffer.append("\tdump                    - " + 
                    "dump captured packets\n");
    if(! suppressed.contains("detail"))
      buffer.append("\tdetail [index]          - " + 
                    "display details on a captured packet\n");
    if(! suppressed.contains("verbose"))
      buffer.append("\tverbose                 - " + 
                    "toggle verbosity level on packet details\n");

    return buffer.toString();
  }


  /**
   * Tester.
   */
  public static void main(String [] args) {
    // main() is overridden because running CaptureConsole without it
    // would invoke the inherited TestConsole's main. 
    System.err.println("use CaptureTool to invoke the console");
  }


  /**
   * The packet capture tool associated with this console.
   */
  private CaptureTool captureTool;

  /**
   * The packet capture system interface.
   */
  private PacketCapture captureSystem;


  // todo: following should be moved into CaptureTool and reference 
  // to PacketCapture removed.

  /**
   * filter expression.
   */
  private String filter = "";

  /**
   * capture data length.
   */
  private int snaplen = PacketCapture.DEFAULT_SNAPLEN;

  /**
   * capture timeout in ms.
   */
  private int timeout = PacketCapture.DEFAULT_TIMEOUT;

  /**
   * whether or not to capture in promiscuous mode.
   */
  private boolean promiscuous = true;


  private String _rcsid = 
    "$Id: CaptureConsole.java,v 1.1 2001/05/17 20:15:36 pcharles Exp $";
}
